import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { BackendQueryFormat } from '../../querybuilder';
import { DatabaseInfo } from '../api';

/** Message format of the error message from the backend */
export type ErrorMessage = {
  errorStatus: any;
  queryID: any;
};

/** Cache type */
export type SessionCacheI = {
  currentDatabase?: string;
  databases: DatabaseInfo[] | undefined;
};

// Define the initial state using that type
export const initialState: SessionCacheI = {
  currentDatabase: undefined,
  databases: undefined,
};

export const sessionSlice = createSlice({
  name: 'session',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    updateCurrentDatabase(state, action: PayloadAction<string|undefined>) {
      state.currentDatabase = action.payload;
    },
    updateDatabaseList(state, action: PayloadAction<DatabaseInfo[]>) {
      console.debug('Updating database list', action);
      state.databases = action.payload;
      if (state.databases.length > 0) {
        const foundDatabase = state.databases.find((db) => db.Name === state.currentDatabase);
        console.log(foundDatabase);
        
        if (!foundDatabase) {
          state.currentDatabase = state.databases[0].Name;
        } else {
          state.currentDatabase = foundDatabase?.Name || undefined;
        }
      }
    },
  },
});

export const { updateCurrentDatabase, updateDatabaseList } = sessionSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const sessionCacheState = (state: RootState) => state.sessionCache;

export default sessionSlice.reducer;
